# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path

from bifrost.tests import base


# NOTE(dtantsur): the module is not importable, hence all the troubles
get_network_data = None  # make pep8 happy
FILE = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                    '..', '..', '..', 'playbooks', 'library',
                    'network_metadata.py')
with open(FILE) as fp:
    # Not going to test the ansible bits, strip the imports away to avoid
    # depending on it.
    script = [line for line in fp if not line.startswith('from ansible.')]
    exec(''.join(script))


TEST_MAC = '12:34:56:78:9a:bc'


class TestBifrostNetworkMetadata(base.TestCase):

    def test_simple(self):
        params = {
            'nics': [{'address': TEST_MAC}],
            'ipv4_address': '1.2.3.4',
            'ipv4_subnet_mask': '255.255.0.0',
        }
        result = get_network_data(params)
        self.assertEqual({
            'links': [
                {'id': TEST_MAC,
                 'ethernet_mac_address': TEST_MAC,
                 'type': 'phy'},
            ],
            'networks': [
                {'id': f'ipv4-{TEST_MAC}',
                 'link': TEST_MAC,
                 'type': 'ipv4',
                 'ip_address': '1.2.3.4',
                 'netmask': '255.255.0.0',
                 'routes': []}
            ],
            'services': [],
        }, result)

    def test_everything(self):
        another_mac = 'aa:aa:aa:bb:cc:dd'
        params = {
            'nics': [{'address': another_mac}, {'address': TEST_MAC}],
            'ipv4_address': '1.2.3.4',
            'ipv4_subnet_mask': '255.255.0.0',
            'ipv4_interface_mac': TEST_MAC,
            'ipv4_gateway': '1.2.1.1',
            'ipv4_nameserver': ['1.1.1.1'],
        }
        result = get_network_data(params)
        self.assertEqual({
            'links': [
                {'id': TEST_MAC,
                 'ethernet_mac_address': TEST_MAC,
                 'type': 'phy'},
            ],
            'networks': [
                {'id': f'ipv4-{TEST_MAC}',
                 'link': TEST_MAC,
                 'type': 'ipv4',
                 'ip_address': '1.2.3.4',
                 'netmask': '255.255.0.0',
                 'dns_nameservers': ['1.1.1.1'],
                 'routes': [{
                     'network': '0.0.0.0',
                     'netmask': '0.0.0.0',
                     'gateway': '1.2.1.1',
                 }]}
            ],
            'services': [{
                'type': 'dns',
                'address': '1.1.1.1',
            }],
        }, result)



# -*- coding: utf-8 -*-

# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import yaml

from bifrost.tests import base
from bifrost.tests import utils


class TestBifrostInventoryFunctional(base.TestCase):

    def setUp(self):
        self.maxDiff = None
        super(TestBifrostInventoryFunctional, self).setUp()

    def test_yaml_to_json_conversion(self):
        # Note(TheJulia) Ultimately this is just ensuring
        # that we get the same output when we pass something
        # in as YAML
        expected_hostvars = """{"hostname1":
 {"uuid": "00000000-0000-0000-0000-000000000002", "driver": "ipmi",
 "name": "hostname1", "ipv4_address": "192.168.1.3", "ansible_ssh_host":
 "192.168.1.3", "provisioning_ipv4_address": "192.168.1.3",
 "driver_info": {"ipmi_address": "192.0.2.3",
 "ipmi_password": "undefined", "ipmi_username": "root",
 "ipmi_target_address": null, "ipmi_target_channel": null,
 "ipmi_transit_address": null, "ipmi_transit_channel": null}, "nics":
 [{"mac": "00:01:02:03:04:06"}], "properties": {"ram": "8192", "cpu_arch":
 "x86_64", "disk_size": "1024", "cpus": "2"}, "host_groups":
 ["baremetal", "nova"]}, "hostname0":
 {"uuid": "00000000-0000-0000-0000-000000000001", "driver": "ipmi",
 "name": "hostname0", "ipv4_address": "192.168.1.2", "ansible_ssh_host":
 "192.168.1.2", "provisioning_ipv4_address": "192.168.1.2",
 "driver_info": {}, "nics":
 [{"mac": "00:01:02:03:04:05"}], "properties": {"ram": "8192",
 "cpu_arch": "x86_64", "disk_size": "512", "cpus": "1"},
 "host_groups": ["baremetal", "nova"]}}""".replace('\n', '')
        (groups, hostvars) = utils.bifrost_data_conversion(
            yaml.safe_dump(json.loads(str(expected_hostvars))))
        del hostvars['127.0.0.1']
        self.assertDictEqual(json.loads(str(expected_hostvars)), hostvars)

    def test_minimal_json(self):
        input_json = """{"h0000-01":{"uuid":
"00000000-0000-0000-0001-bad00000010","name":"h0000-01","driver_info"
:{"ipmi_address":"10.0.0.78","ipmi_username":"ADMIN","
ipmi_password":"ADMIN"},"driver":"ipmi"}}""".replace('\n', '')
        expected_json = """{"h0000-01":{"uuid":
"00000000-0000-0000-0001-bad00000010","name":"h0000-01","driver_info"
:{"ipmi_address":"10.0.0.78","ipmi_username":"ADMIN","
ipmi_password":"ADMIN"},"driver":"ipmi","addressing_mode":
"dhcp","host_groups": ["baremetal"]}}""".replace('\n', '')
        (groups, hostvars) = utils.bifrost_data_conversion(input_json)
        del hostvars['127.0.0.1']
        self.assertDictEqual(json.loads(str(expected_json)), hostvars)



#!/usr/bin/env python
# coding: utf-8 -*-

# (c) 2015, Hewlett-Packard Development Company, L.P.
#
# This module is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software.  If not, see <http://www.gnu.org/licenses/>.

DOCUMENTATION = '''
---
module: network_metadata
short_description: Returns a config-drive network-metadata dictionary
extends_documentation_fragment: openstack
'''


def get_network_data(params):
    links = []
    networks = []

    if params.get('ipv4_interface_mac'):
        nic_id = params['ipv4_interface_mac']

        if params.get('vlan_id'):
            nic_id = 'vlan-%s' % params['ipv4_interface_mac']

            links.append({
                'id': nic_id,
                'type': 'vlan',
                'vlan_id': params['vlan_id'],
                'vlan_link': params['ipv4_interface_mac'],
                'vlan_mac_address': params['ipv4_interface_mac'],
            })

        link = {
            'id': params['ipv4_interface_mac'],
            'type': 'phy',
            'ethernet_mac_address': params['ipv4_interface_mac'],
        }
        if params.get('network_mtu'):
            link['mtu'] = params['network_mtu']
        links.append(link)

        for nic in params['nics']:
            if nic['address'] == params['ipv4_interface_mac']:
                network = {
                    'id': 'ipv4-%s' % nic_id,
                    'link': nic_id,
                    'type': 'ipv4',
                    'ip_address': params['ipv4_address'],
                    'netmask': params['ipv4_subnet_mask'],
                    'routes': []
                }
                if params.get('ipv4_nameserver'):
                    network['dns_nameservers'] = \
                        params['ipv4_nameserver']
                if params.get('ipv4_gateway'):
                    network['routes'].append({
                        'network': '0.0.0.0',
                        'netmask': '0.0.0.0',
                        'gateway': params['ipv4_gateway']
                    })
                networks.append(network)
    else:
        for i, nic in enumerate(params['nics']):
            nic_id = nic['address']
            if params.get('vlan_id'):
                nic_id = 'vlan-%s' % nic['address']

                links.append({
                    'id': nic_id,
                    'type': 'vlan',
                    'vlan_id': params['vlan_id'],
                    'vlan_link': nic['address'],
                    'vlan_mac_address': nic['address']
                })

            link = {
                'id': nic['address'],
                'type': 'phy',
                'ethernet_mac_address': nic['address'],
            }
            if params.get('network_mtu'):
                link['mtu'] = params['network_mtu']
            links.append(link)

            if i == 0:
                network = {
                    'id': 'ipv4-%s' % nic_id,
                    'link': nic_id,
                    'type': 'ipv4',
                    'ip_address': params['ipv4_address'],
                    'netmask': params['ipv4_subnet_mask'],
                    'routes': []
                }
                if params.get('ipv4_nameserver'):
                    network['dns_nameservers'] = \
                        params['ipv4_nameserver']
                if params.get('ipv4_gateway'):
                    network['routes'].append({
                        'network': '0.0.0.0',
                        'netmask': '0.0.0.0',
                        'gateway': params['ipv4_gateway']
                    })
                networks.append(network)
            else:
                networks.append({
                    'id': 'ipv4-dhcp-%s' % nic_id,
                    'link': nic_id,
                    'type': 'ipv4_dhcp',
                })

    services = []
    if params.get('ipv4_nameserver'):
        for item in params['ipv4_nameserver']:
            services.append({
                'type': 'dns',
                'address': item
            })

    return {
        'links': links,
        'networks': networks,
        'services': services
    }


def main():
    argument_spec = dict(
        ipv4_address=dict(required=False),
        ipv4_gateway=dict(required=False),
        ipv4_interface_mac=dict(required=False),
        ipv4_nameserver=dict(required=False, type='list'),
        ipv4_subnet_mask=dict(required=False),
        vlan_id=dict(required=False),
        network_mtu=dict(required=False),
        nics=dict(required=False, type='list'),
        node_network_data=dict(required=False, type='dict')
    )

    module = AnsibleModule(argument_spec)  # noqa: F405

    network_metadata = module.params['node_network_data']
    if not network_metadata:
        network_metadata = get_network_data(module.params)
    facts = {'network_metadata': network_metadata}

    module.exit_json(changed=False, ansible_facts=facts)


# this is magic, see lib/ansible/module_common.py
from ansible.module_utils.basic import *  # noqa: E402

if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os.path
import shutil
import tempfile
from unittest import mock

from bifrost import cli
from bifrost.tests import base


FILE_NO_BRANCH = """
[gerrit]
host=review.opendev.org
port=29418
project=openstack/bifrost.git
"""


FILE_WITH_BRANCH = """
[gerrit]
host=review.opendev.org
port=29418
project=openstack/bifrost.git
defaultbranch=stable/banana
"""


class TestGetRelease(base.TestCase):
    def setUp(self):
        super().setUp()
        self.temp_dir = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(self.temp_dir))

    def test_provided(self):
        self.assertEqual('stable/ussuri', cli.get_release('ussuri'))
        self.assertEqual('stable/ussuri', cli.get_release('stable/ussuri'))
        self.assertEqual('master', cli.get_release('master'))

    @mock.patch.object(cli, 'BASE', '/non/existing/dir')
    def test_no_file(self):
        self.assertIsNone(cli.get_release(None))

    def test_from_file_no_branch(self):
        with open(os.path.join(self.temp_dir, '.gitreview'), 'wt') as fp:
            fp.write(FILE_NO_BRANCH)
        with mock.patch.object(cli, 'BASE', self.temp_dir):
            self.assertIsNone(cli.get_release(None))

    def test_from_file_with_branch(self):
        with open(os.path.join(self.temp_dir, '.gitreview'), 'wt') as fp:
            fp.write(FILE_WITH_BRANCH)
        with mock.patch.object(cli, 'BASE', self.temp_dir):
            self.assertEqual('stable/banana', cli.get_release(None))
